;--------------------------------------------------------------------
; Description:
; This subroutine loades the nametable and attributes into the PPU from a 1024 byte block
;
; VARIABLES:
; **********************************************************************************
; PTR.Nametable 			holds the address location in ROM where the nametable is located.
; VAR.NametableDestination	defines which one of the PPU nametable's will be written to (0, 1, 2, 3)
; **********************************************************************************
;
; Before calling this subroutine, you will first need to set the two variables described above.
; Below is an example of how they could be done prior to a JSR command
;
; LOADING CODE EXAMPLE:
; **********************************************************************************
; LDA #$00
; STA VAR.NametableDestination
; LDA #<DAT.SceneTITLE.Background.Nametable
; STA PTR.Nametable
; LDA #>DAT.SceneTITLE.Background.Nametable
; STA PTR.Nametable + 1
; JSR LBL.PPULoadNametable

LBL.PPULoadNametable
  
  ; Calculates the PPU Address High Byte based on the value of VAR.NametableDestination
  ; The PPU nametables start had a high byte address of #$20, #$24, #$28, #$2C
  CLC
  LDA VAR.NametableDestination
  ASL A
  ASL A
  ADC #$20
  TAX
  
  ; Resets the PPU address
  LDA $2002
  
  ; Sets the PPU Address High Byte
  STX $2006
  
  ; Sets the PPU Address Low Byte
  LDA #$00
  STA $2006

  ; Copies data 1 byte of a time.
  LDX #$04
  LDY #$00
LBL.Nametable.Loop:
    LDA (PTR.Nametable), y
    STA $2007
    INY
    BNE LBL.Nametable.Loop
      INC PTR.Nametable + 1;
      DEX
      BNE LBL.Nametable.Loop
RTS