; Description:
; This subroutine reads from the first player controller
; port to determine which buttons have been pressed or released since the
; last executing.  The results are stored into four variables that must be defined
; in the engine header (preferably on the Zero Page)
; 
; VARIABLES:
; **********************************************************************************
; VAR.Controller          holds the current state of the buttons
; VAR.Controller.Old      holds the prior state of the buttons
; VAR.Controller.Released holds buttons that have have change from on to off
; VAR.Controller.Pressed  holds buttons that have been pressed from off to on
; **********************************************************************************
; 
; These variables are encoded into a single byte with the rule:
;--------------------------------------------------------------------
; bit:       7     6     5     4     3     2     1     0
; button:    A     B   select start  up   down  left right
;--------------------------------------------------------------------
;
;  This subroutine can be accessed at any time through a call to 
;  JSR LBL.ReadControllers.  Not other loading work is necessary

LBL.ReadController:
  
  ;  Sets up the port for reading
  LDA #$01
  STA CONTROLLER
  LDA #$00
  STA CONTROLLER
  
  ; Initializes counter.
  LDX #$08

  ; Sets the Old variables
  LDA VAR.Controller;
  STA VAR.Controller.Old;
  
  ; Sets the Current variable state
  ; 8 reads from the $4016 port are required to get all the button states
  ; this uses a combination of bit shifting and the carry bit to merge their
  ; values into a single byte.
LBL.ReadController.Loop:
  LDA CONTROLLER
  LSR A
  ROL VAR.Controller
  DEX
  BNE LBL.ReadController.Loop

  ;  Sets the Released variable
  LDA VAR.Controller
  EOR #$FF
  AND VAR.Controller.Old
  STA VAR.Controller.Released

  ; Sets the Pressed variable
  LDA VAR.Controller.Old
  EOR #$FF
  AND VAR.Controller
  STA VAR.Controller.Pressed

RTS  