;----------------------------------------------------------------
; COMPILER CONSTANTS:
;----------------------------------------------------------------
PRGSIZE = 512
PRGCOUNT = PRGSIZE / 16

CHRSIZE = 256
CHRCOUNT = CHRSIZE / 8

PRGBANKCOUNTER = 0
CHRBANKCOUNTER = 0

FALSE     EQU #$00
TRUE	  EQU #$01

PPUCTRL	  EQU #$2000
PPUMASK   EQU #$2001
PPUSTATUS EQU #$2002
OAMADDR   EQU #$2003
OAMDATA   EQU #$2004
PPUSCROLL EQU #$2005
PPUADDR   EQU #$2006
PPUDATA   EQU #$2007
OAMDMA    EQU #$4014

BANKSELECT  EQU #$8000
BANKDATA	EQU #$8001

CONTROLLER  EQU #$4016

;----------------------------------------------------------------
; iNES header
;----------------------------------------------------------------

   .db "NES", $1a 			;identification of the iNES header
   .db PRGCOUNT 			;number of 16KB PRG-ROM pages
   .db CHRCOUNT				;number of 8KB CHR-ROM pages
   .db $40
   .dsb 9, $00 				;clear the remaining bytes

;----------------------------------------------------------------
; VARIABLES:
;----------------------------------------------------------------
include "L5RMemoryMap.asm"

;----------------------------------------------------------------
; Program Bank 00
;----------------------------------------------------------------
PRGBANK00 = PRGBANKCOUNTER
PRGBANKCOUNTER = PRGBANKCOUNTER + 1

.include "TITLE/TitleScreen.asm"

;----------------------------------------------------------------
; Program Bank Unused
;----------------------------------------------------------------
.rept (PRGCOUNT - PRGBANKCOUNTER - 1)
  .base $8000
  .pad  $C000
.endr

;----------------------------------------------------------------
; Tile Bank Unused
;----------------------------------------------------------------
.rept(CHRSIZE - CHRBANKCOUNTER)
  .base $1000
  .pad  $1400
.endr

;----------------------------------------------------------------
; Program Bank Fixed 1 - $C000 to $0DFF
;----------------------------------------------------------------
PRGBANKFIXED1 = PRGBANKCOUNTER
PRGBANKCOUNTER = PRGBANKCOUNTER + 1

.base $C000

include "ENGINE/ClearCPURAM.asm"
include "ENGINE/ClearSpriteBuffer.asm"
include "ENGINE/LoadNametable.asm"
include "ENGINE/ReadControllerPlayerOne.asm"
include "ENGINE/NMI.asm"

.pad  $E000

;----------------------------------------------------------------
; Program Bank Fixed 2 - $E000 to $FFFF
;----------------------------------------------------------------
PRGBANKFIXED2 = PRGBANKCOUNTER

.base $E000

LBL.RESET:
  SEI          ; disable IRQs
  CLD          ; disable decimal mode

  LDX #$40
  STX $4017    ; disable APU frame IRQ
  
  LDX #$00
  STX PPUCTRL    ; disable NMI
  STX PPUMASK    ; disable rendering
  STX $4010      ; disable DMC IRQs

  JSR LBL.ResetCPURAM
  JSR LBL.ResetSprites
  
  - BIT PPUSTATUS
    BPL -
  
  - BIT PPUSTATUS
    BPL -

  ; Set PPUCTRL to enable NMI (bit 7) and use CHR bank 1 (bit 3) for sprites
  LDA #%10001000
  STA VAR.PPUCTRL
  STA PPUCTRL
  
  ; Sets the Game State
  LDA #STATE.START	
  STA VAR.GameState
  
  ; Loads the initial PRG banks into memory (Title Screen)
  LDA #$06
  STA BANKSELECT
  LDA #(2*PRGBANK00)
  STA BANKDATA
  LDA #$07
  STA BANKSELECT
  LDA #(2*PRGBANK00+1)
  STA BANKDATA  
  
; Main game loop
LBL.Main: 
  JSR LBL.GameStateJumper
  JMP LBL.Main

;----------------------------------------------------------------
; MAIN JUMPER
;----------------------------------------------------------------
; Description:
; These are a special set of subroutines (or JMP-Tables) that uses a
; table of label addresses to quickly jump to a block of code.
; This jumper directs control the the correct game state block.

LBL.GameStateJumper:
  LDA VAR.GameState
  ASL
  TAX
  LDA TBL.GameState + 1, x
  PHA
  LDA TBL.GameState, x
  PHA
  RTS

;----------------------------------------------------------------
; MAIN JUMPER TABLE
;----------------------------------------------------------------
; Indicates the state where the game starts up in
STATE.START						 = STATE.TITLE.Initialize

; List of the game states
STATE.TITLE.Initialize           = 0
STATE.TITLE.Update				 = 1

; Table of addresses used in the main jumper.  Subtracting 1 from the addresses are REQUIRED.
TBL.GameState:
 .dw LBL.TITLE.Initialize - 1
 .dw LBL.TITLE.Update - 1

.pad $FFFA
 
;----------------------------------------------------------------
; interrupt vectors
;----------------------------------------------------------------

   .org $FFFA

   .dw LBL.NMI
   .dw LBL.RESET
   .dw 0
